@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Document::OpenAPI;
 # ABSTRACT: One OpenAPI v3.1 document
 # KEYWORDS: JSON Schema data validation request response OpenAPI
 
-our $VERSION = '0.050';
+our $VERSION = '0.051';
 
 use 5.020;
 use Moo;
@@ -21,7 +21,6 @@ use File::ShareDir 'dist_dir';
 use Path::Tiny;
 use List::Util 'pairs';
 use Ref::Util 'is_plain_hashref';
-use MooX::HandlesVia;
 use MooX::TypeTiny 0.002002;
 use Types::Standard qw(InstanceOf HashRef Str Enum);
 use namespace::clean;
@@ -58,35 +57,20 @@ has json_schema_dialect => (
   coerce => sub { $_[0]->$_isa('Mojo::URL') ? $_[0] : Mojo::URL->new($_[0]) },
 );
 
-my $reffable_entities = Enum[qw(response parameter example request-body header security-scheme link callbacks path-item)];
-
-# json pointer => entity name
-has entities => (
-  is => 'bare',
-  isa => HashRef[$reffable_entities],
-   handles_via => 'Hash',
-   handles => {
-     _add_entity_location => 'set',
-     get_entity_at_location => 'get',
-  },
-  lazy => 1,
-  default => sub { {} },
-
-);
+# json pointer => entity name (indexed by integer); overrides parent
+sub __entities { qw(schema response parameter example request-body header security-scheme link callbacks path-item) }
 
 # operationId => document path
-has operationIds => (
-  is => 'bare',
+has _operationIds => (
+  is => 'ro',
   isa => HashRef[Str],
-   handles_via => 'Hash',
-   handles => {
-     _add_operationId => 'set',
-     get_operationId_path => 'get',
-  },
   lazy => 1,
   default => sub { {} },
 );
 
+sub get_operationId_path { $_[0]->_operationIds->{$_[1]} }
+sub _add_operationId { $_[0]->_operationIds->{$_[1]} = Str->($_[2]) }
+
 sub traverse ($self, $evaluator) {
   $self->_add_vocab_and_default_schemas;
 
@@ -102,6 +86,7 @@ sub traverse ($self, $evaluator) {
     configs => {},
     spec_version => $evaluator->SPECIFICATION_VERSION_DEFAULT,
     vocabularies => [],
+    subschemas => [],
   };
 
   # this is an abridged form of https://spec.openapis.org/oas/3.1/schema/latest
@@ -176,9 +161,8 @@ sub traverse ($self, $evaluator) {
           return 1;
         },
         '$ref' => sub ($data, $schema, $state) {
-          my ($entity) = ($schema->{'$ref'} =~ m{#/\$defs/([^/]+?)(?:-or-reference)?$});
-          $self->_add_entity_location($state->{data_path}, $entity)
-            if $entity and $reffable_entities->check($entity);
+          my ($entity) = ($schema->{'$ref'} =~ m{#/\$defs/([^/]+?)(?:-or-reference)$});
+          $self->_add_entity_location($state->{data_path}, $entity) if $entity;
 
           push @operation_paths, [ $data->{operationId} => $state->{data_path} ]
             if $schema->{'$ref'} eq '#/$defs/operation' and defined $data->{operationId};
@@ -188,7 +172,6 @@ sub traverse ($self, $evaluator) {
       },
     },
   );
-  $self->_add_entity_location($_, 'schema') foreach @json_schema_paths;
 
   if (not $result) {
     $_->mode('evaluate') foreach $result->errors;
@@ -228,6 +211,8 @@ sub traverse ($self, $evaluator) {
     $self->_traverse_schema($self->get($json_schema_paths[$idx]), { %$state, schema_path => $json_schema_paths[$idx] });
   }
 
+  $self->_add_entity_location($_, 'schema') foreach $state->{subschemas}->@*;
+
   foreach my $pair (@operation_paths) {
     my ($operation_id, $path) = @$pair;
     if (my $existing = $self->get_operationId_path($operation_id)) {
@@ -295,11 +280,12 @@ sub _traverse_schema ($self, $schema, $state) {
   return if $subschema_state->{errors}->@*;
 
   push $state->{identifiers}->@*, $subschema_state->{identifiers}->@*;
+  push $state->{subschemas}->@*, $subschema_state->{subschemas}->@*;
 }
 
 # callback hook for Sereal::Decoder
 sub THAW ($class, $serializer, $data) {
-  foreach my $attr (qw(schema evaluator entities)) {
+  foreach my $attr (qw(schema evaluator _entities)) {
     die "serialization missing attribute '$attr': perhaps your serialized data was produced for an older version of $class?"
       if not exists $class->{$attr};
   }
@@ -320,7 +306,7 @@ JSON::Schema::Modern::Document::OpenAPI - One OpenAPI v3.1 document
 
 =head1 VERSION
 
-version 0.050
+version 0.051
 
 =head1 SYNOPSIS
 
@@ -345,7 +331,7 @@ C<https://spec.openapis.org/oas/3.1/schema-base/latest> (an alias for the latest
 
 and the L<OpenAPI v3.1 specification|https://spec.openapis.org/oas/v3.1.0>.
 
-=for Pod::Coverage THAW
+=for Pod::Coverage THAW get_entity_at_location
 
 =head1 ATTRIBUTES
 
