@@ -1,10 +1,10 @@
 use strictures 2;
-package OpenAPI::Modern; # git description: v0.049-6-g5704d3c
+package OpenAPI::Modern; # git description: v0.050-8-g0e8d7b2
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI v3.1 document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI v3.1 Swagger HTTP request response
 
-our $VERSION = '0.050';
+our $VERSION = '0.051';
 
 use 5.020;
 use utf8;
@@ -156,8 +156,10 @@ sub validate_request ($self, $request, $options = {}) {
 
     $state->{schema_path} = jsonp($state->{schema_path}, $method);
 
-    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header') },
-        'RFC9112 §6.2-2: A sender MUST NOT send a Content-Length header field in any message that contains a Transfer-Encoding header field')
+    # RFC9112 §6.2-2: A sender MUST NOT send a Content-Length header field in any message that
+    # contains a Transfer-Encoding header field.
+    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header', 'Content-Length') },
+        'Content-Length cannot appear together with Transfer-Encoding')
       if defined $request->headers->content_length and $request->content->is_chunked;
 
     # RFC9112 §6.3-7: A user agent that sends a request that contains a message body MUST send
@@ -253,10 +255,18 @@ sub validate_response ($self, $response, $options = {}) {
         if $response->is_success and $method eq 'connect';
     }
 
-    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header') },
-        'RFC9112 §6.2-2: A sender MUST NOT send a Content-Length header field in any message that contains a Transfer-Encoding header field')
+    # RFC9112 §6.2-2: A sender MUST NOT send a Content-Length header field in any message that
+    # contains a Transfer-Encoding header field.
+    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header', 'Content-Length') },
+        'Content-Length cannot appear together with Transfer-Encoding')
       if defined $response->headers->content_length and $response->content->is_chunked;
 
+    # RFC9112 §6.3-7: A user agent that sends a request that contains a message body MUST send
+    # either a valid Content-Length header field or use the chunked transfer coding.
+    ()= E({ %$state, data_path => jsonp($state->{data_path}, 'header') }, 'missing header: Content-Length')
+      if $response->body_size and not $response->headers->content_length
+        and not $response->content->is_chunked;
+
     my $response_name = first { exists $operation->{responses}{$_} }
       $response->code, substr(sprintf('%03s', $response->code), 0, -2).'XX', 'default';
 
@@ -461,12 +471,22 @@ sub recursive_get ($self, $uri_reference) {
   my $ref = $uri_reference;
   my ($depth, $schema);
 
+  my $entity_type;
+  $entity_type = $self->openapi_document->get_entity_at_location(Mojo::URL->new($uri_reference)->fragment)
+    if $uri_reference =~ /^#/;
+
   while ($ref) {
     die 'maximum evaluation depth exceeded' if $depth++ > $self->evaluator->max_traversal_depth;
     my $uri = Mojo::URL->new($ref)->to_abs($base);
 
     my $schema_info = $self->evaluator->_fetch_from_uri($uri);
+
     die('unable to find resource ', $uri) if not $schema_info;
+    die sprintf('bad $ref to %s: not a "%s"', $schema_info->{canonical_uri}, $entity_type)
+      if $entity_type and $entity_type ne 'schema'
+        and $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne $entity_type;
+
+    $entity_type //= $schema_info->{document}->get_entity_at_location($schema_info->{document_path});
     $schema = $schema_info->{schema};
     $base = $schema_info->{canonical_uri};
     $ref = $schema->{'$ref'};
@@ -709,7 +729,7 @@ sub _resolve_ref ($self, $entity_type, $ref, $state) {
     if $state->{depth}++ > $self->evaluator->max_traversal_depth;
 
   abort({ %$state, keyword => '$ref' }, 'EXCEPTION: bad $ref to %s: not a "%s"', $schema_info->{canonical_uri}, $entity_type)
-    if ($schema_info->{document}->get_entity_at_location($schema_info->{document_path})//'') ne $entity_type;
+    if $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne $entity_type;
 
   $state->{initial_schema_uri} = $schema_info->{canonical_uri};
   $state->{traversed_schema_path} = $state->{traversed_schema_path}.$state->{schema_path}.jsonp('/$ref');
@@ -771,10 +791,8 @@ sub _convert_request ($request) {
   return $request if $request->isa('Mojo::Message::Request');
   if ($request->isa('HTTP::Request')) {
     my $req = Mojo::Message::Request->new;
-    if (not defined $request->headers->content_length) {
-      my $length = length $request->content_ref->$*;
-      $req->headers->content_length($length) if $length;
-    }
+    # we could call $req->fix_headers here to add a missing Content-Length, but proper requests from
+    # the network should always have it set.
     $req->parse($request->as_string);
     return $req;
   }
@@ -786,11 +804,9 @@ sub _convert_response ($response) {
   return $response if $response->isa('Mojo::Message::Response');
   if ($response->isa('HTTP::Response')) {
     my $res = Mojo::Message::Response->new;
-    if (not defined $response->headers->content_length) {
-      my $length = length $response->content_ref->$*;
-      $res->headers->content_length($length) if $length;
-    }
     $res->parse($response->as_string);
+    # we could call $req->fix_headers here to add a missing Content-Length, but proper requests from
+    # the network should always have it set.
     return $res;
   }
   croak 'unknown type '.ref($response);
@@ -819,7 +835,7 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI v3.1 d
 
 =head1 VERSION
 
-version 0.050
+version 0.051
 
 =head1 SYNOPSIS
 
